/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/posix.hpp

    Copyright © 2014, 2016 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    ---------------------------------------------------------------------- copyright and license ---
*/


#ifndef _POSIX_HPP_
#define _POSIX_HPP_


#include "config.h"
#include "base.hpp"

#include <stdexcept>    // std::runtime_error

#include <pthread.h>
#include <signal.h>


namespace posix {


    string_t get_env( string_t const & name );
    string_t base_name( string_t const & path );
    string_t syserrmsg( int error );


    // ---------------------------------------------------------------------------------------------
    // error_t
    // ---------------------------------------------------------------------------------------------

    class error_t : public std::runtime_error {

        public :

            explicit error_t( string_t const & what );
            explicit error_t( string_t const & what, int error );

    }; // class error_t


    // ---------------------------------------------------------------------------------------------
    // signal_t
    // ---------------------------------------------------------------------------------------------


    namespace signal {

        class set_t : public object_t {

            public :

                set_t();

                void  clear();

                set_t & operator +=( int signo );

            public :

                sigset_t rep;

        }; // class set_t


        set_t mask( int how, set_t const & set );
        int   wait( set_t const & set );
        void  kill( pid_t pid, int signal );

    }; // namespace signal


    // ---------------------------------------------------------------------------------------------
    // mutex_t
    // ---------------------------------------------------------------------------------------------


    class mutex_t : public non_copyable_object_t {

        public :

            explicit mutex_t();
            virtual ~mutex_t();

            void lock();
            void unlock();

        public :

            class locker_t : public non_copyable_object_t {
                public :
                    explicit locker_t( mutex_t & mutex );
                    virtual ~locker_t();
                private :
                    mutex_t & _mutex;
            }; // class locker_t

        private :

            pthread_mutex_t _handle;

    }; // class mutex_t


    // ---------------------------------------------------------------------------------------------
    // thread_t
    // ---------------------------------------------------------------------------------------------


    class thread_t : public non_copyable_object_t {

        public :

            explicit thread_t();
            virtual ~thread_t();

            void start();
            void join();

            static unsigned int gettid();

        protected :

            virtual void body() = 0;

        private :

            static void * _body( void * arg );

            enum state_t {
                inited,
                started,
                joined
            }; // enum state_t

            state_t         _state;
            pthread_t       _handle;

        protected :

            string_t        _what;

    }; // class thread_t


}; // namespace posix


#endif // _POSIX_HPP_

// end of file //
