/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/strings.cpp

    Copyright © 2014, 2016 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    ---------------------------------------------------------------------- copyright and license ---
*/


#include "string.hpp"

#include <algorithm>


string_pair_t
split(
    char             chr,
    string_t const & str
) {
    string_pair_t result;
    size_t pos = str.find( chr );
    if ( pos == string_t::npos ) {
        result.first  = str;
    } else {
        result.first  = str.substr( 0, pos );
        result.second = str.substr( pos + 1 );
    }; // if
    return result;
}; // split


ssize_t
find(
    string_t const & needle,
    strings_t const & haystack
) {
    strings_t::const_iterator it = std::find( haystack.begin(), haystack.end(), needle );
    // TODO: Make sure result fits to ssize_t.
    return it == haystack.end() ? -1 : it - haystack.begin();
}; // find


string_t
left(
    string_t const & str,
    size_t           len
) {
    return str.substr( 0, len );
}; // left


string_t
right(
    string_t const & str,
    size_t           len
) {
    return str.substr( str.size() >= len ? str.size() - len : 0, len );
}; // right


bool
natural_number(
    string_t const & str
) {
    return str.find_first_not_of( "0123456789" ) == string_t::npos;
}; // is_natural_number


string_t
uc(
    string_t const & str
) {
    string_t res;
    res.reserve( str.size() );
    for ( size_t i = 0; i < str.size(); ++ i ) {
        res += toupper( str[ i ] );
    };
    return res;
}; // uc


string_t
chop(
    string_t const & str,
    string_t const & sfx
) {
    if ( right( str, sfx.size() ) == sfx ) {
        return left( str, str.size() - sfx.size() );
    } else {
        return str;
    };
};


// end of file //
