/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/strings.hpp

    Copyright © 2014, 2016 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    ---------------------------------------------------------------------- copyright and license ---
*/


#ifndef _STRING_HPP_
#define _STRING_HPP_


#include "config.h"

#include <sstream>
#include <stdexcept>
#include <string>
#include <utility>      // std::pair
#include <vector>

#include <unistd.h>     // ssize_t


/**
    Shorter alias for frequently used type.

    Yeah, `string_t` is shorter than `std::string`. :-)
*/
typedef std::string     string_t;

/**
    Shorter alias for frequently used type.
*/
typedef std::vector< string_t >     strings_t;

/**
    Shorter alias for frequently used type.
*/
typedef std::pair< string_t, string_t > string_pair_t;


/**
    Utility macro.
*/
#if 1
    #define STR( _ARGS_ ) reinterpret_cast< std::ostringstream & >( std::ostringstream().flush() << _ARGS_ ).str()
#else
    #define STR( _ARGS_ ) ( { std::ostringstream _ostr_; _ostr_ << _ARGS_; _ostr_.str(); } )
#endif


string_pair_t   split( char chr, string_t const & str );
ssize_t         find( string_t const & needle, strings_t const & haystack );
string_t        left( string_t const & str, size_t len );
string_t        right( string_t const & str, size_t len );
bool            natural_number( string_t const & str );
string_t        uc( string_t const & str );
string_t        chop( string_t const & str, string_t const & sfx );


template<
    typename type_t
>
type_t
val(
    string_t const & str
) {
    std::istringstream istr( str );
    type_t value = 0;
    istr >> std::noskipws >> value;
    if ( istr.fail() or istr.bad() ) {
        throw std::runtime_error( "Parsing failed" );
    }; // if
    // Now try to read one more character. We expect failure and EOF.
    char ch;
    istr >> std::noskipws >> ch;
    if ( not ( istr.fail() and istr.eof() ) ) {
        throw std::runtime_error( "Parsing failed" );
    }; // if
    return value;
}; // val


#endif // _STRING_HPP_

// end of file //
