/*
    ---------------------------------------------------------------------- copyright and license ---

    file: src/tapper.hpp

    Copyright © 2014, 2016 Van de Bugger.

    This file is a part of Tapper.

    Tapper is free software: you can redistribute it and/or modify it under the terms of the GNU
    General Public License as published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Tapper is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
    even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License along with Tapper.  If not,
    see <http://www.gnu.org/licenses/>.

    ---------------------------------------------------------------------- copyright and license ---
*/


#ifndef _TAPPER_HPP_
#define _TAPPER_HPP_


#include "base.hpp"
#include "posix.hpp"
#include "x.hpp"

#include <map>
#include <vector>


// -------------------------------------------------------------------------------------------------
// tapper_t
// -------------------------------------------------------------------------------------------------


class tapper_t : public non_copyable_object_t {

    public :

        explicit tapper_t( x::kb_t & kb );
        virtual ~tapper_t();
        void start();
        void stop();

    protected :

        virtual void on_tap( KeyCode key_code ) = 0;

    private :

        class interceptor_t : public x::record::context_t {

            public :

                explicit interceptor_t( tapper_t & _master );

            protected :

                virtual void on_intercept( XRecordInterceptData * data );

            private :

                typedef std::vector< bool > kb_state_t;

                tapper_t &          master;
                kb_state_t          state;      // Map of keys: true == key is currently pressed.
                int                 pressed_keys;       // Number of currently pressed keys.
                int                 last_pressed_key;   // ~Code of last pressed key.
                Time                pressed_at;         // ~Time of last key press event.

        }; // interceptor

    protected :

        x::kb_t & _kb;

    private :

        interceptor_t               interceptor;
        x::kb_t::autorepeat_rate_t  autorepeat_rate;

}; // class tapper_t


#endif // _TAPPER_HPP_

// end of file //
